<?php
/**

 *
 * Clase para Generar Factura en Formato XML
 */
require_once 'ubl/UBLNamespaces.php';
require_once 'ubl/UBLSunat.php';

class FacturaXML extends UBLSunat
{

    public function __construct($data, $dsSignature = false)
    {
        $this->namespaces = UBLNamespaces::Invoice();

        $montos    = $data['montos'];
        $notas     = $data['notas'];
        $impuestos = $data['impuestos'];
        $items     = $data['items'];

        $this->dom  = new DOMDocument('1.0', 'UTF-8');
        $this->root = $this->dom->createElementNS(UBLNamespaces::$xmlnsInvoice, 'Invoice');

        foreach ($this->namespaces as $pfx => $namespace) {
            $this->root->setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns:' . $pfx, $namespace);
        }

        $this->dom->appendChild($this->root);
        $this->current = $this->root;

        $this->append_fw('ext:UBLExtensions');
        $this->append_fw('ext:UBLExtension')
            ->append_fw('ext:ExtensionContent')
            ->append_fw('sac:AdditionalInformation');

        foreach ($montos as $key => $monto) {
            if (!isset($monto['nombre'])) {
                $this->append_fw('sac:AdditionalMonetaryTotal')
                    ->append('cbc:ID', $monto['id']);
            } else {
                $this->append_fw('sac:AdditionalMonetaryTotal')
                    ->append('cbc:ID', $monto['id'])
                    ->append_nnv('cbc:Name', $monto['nombre']);
            }
            if (isset($monto['valor']['referencia'])) {
                $this->append_nnv('sac:ReferenceAmount', $monto['valor']['referencia'])->attribute('currencyID', $data['documento']['moneda']);
            }
            if (isset($monto['valor']['pagable'])) {
                $this->append('cbc:PayableAmount', $monto['valor']['pagable'])->attribute('currencyID', $data['documento']['moneda']);
            }
            if (isset($monto['porcentaje'])) {
                $this->append_nnv('cbc:Percent', $monto['porcentaje']);
            }
            if (isset($monto['valor']['total'])) {
                $this->append_nnv('sac:TotalAmount', $monto['valor']['total'])->attribute('currencyID', $data['documento']['moneda']);
            }
            $this->pop();
        }

        foreach ($notas as $key => $nota) {
            if (!isset($nota['nombre'])) {
                $this->append_fw('sac:AdditionalProperty')
                    ->append('cbc:ID', $nota['id'])
                    ->append('cbc:Value', $nota['valor'])
                    ->pop();
            } else {
                $this->append_fw('sac:AdditionalProperty')
                    ->append('cbc:ID', $nota['id'])
                    ->append_nnv('cbc:Name', $nota['nombre'])
                    ->append('cbc:Value', $nota['valor'])
                    ->pop();
            }
        }

        //TODO AGREGAR DATOS DE FACTURA GUIA
        if (isset($data['documento']['tipo_transaccion'])) {
            $this
                ->append_fw('sac:SUNATTransaction')
                ->append('cbc:ID', $data['documento']['tipo_transaccion'])
                ->pop();
        }

        $this->pop()->pop()->pop();

        // En esta zona va el certificado digital  - (Firma Digital).
        if ($dsSignature) {
            $this
                ->append_fw('ext:UBLExtension')
                ->append_fw('ext:ExtensionContent', ' ');
        }

        $this
            ->reset()
            //cabecera UBL
            ->append('cbc:UBLVersionID', '2.0')
            ->append('cbc:CustomizationID', '1.0')

            //datos del documento
            ->append('cbc:ID', $data['documento']['numero'])
            ->append('cbc:IssueDate', $data['documento']['fecha_emision'])
            ->append('cbc:InvoiceTypeCode', $data['documento']['tipo_factura'])
            ->append('cbc:DocumentCurrencyCode', $data['documento']['moneda'])

            //datos del firmante
            ->append_fw('cac:Signature')
            ->append('cbc:ID', 'IDSignKG')
            ->append('cbc:Note', $data['emisor']['nota'])
            ->append('cbc:ValidatorID', $data['emisor']['codigo'])
            ->append_fw('cac:SignatoryParty')
            ->append_fw('cac:PartyIdentification')->append('cbc:ID', $data['emisor']['documento']['numero'])->pop()
            ->append_fw('cac:PartyName')->append('cbc:Name', $data['emisor']['datos']['razon_social'], true)->pop()
            ->pop()
            ->append_fw('cac:DigitalSignatureAttachment')->append_fw('cac:ExternalReference')->append('cbc:URI', '#signatureKG')->pop(2)
            ->reset()

            //datos del emisor
            ->append_fw('cac:AccountingSupplierParty')
            ->append('cbc:CustomerAssignedAccountID', $data['emisor']['documento']['numero'])
            ->append('cbc:AdditionalAccountID', $data['emisor']['documento']['tipo'])
            ->append_fw('cac:Party')
            ->append_fw('cac:PartyName')
            ->append('cbc:Name', $data['emisor']['datos']['nombre_comercial'], true)
            ->pop()
            ->append_fw('cac:PostalAddress')
            ->append_nnv('cbc:ID', $data['emisor']['ubicacion']['ubigeo'])
            ->append_nnv('cbc:StreetName', $data['emisor']['ubicacion']['direccion'])
            ->append_nnv('cbc:CitySubdivisionName', $data['emisor']['ubicacion']['urbanizacion'])
            ->append_nnv('cbc:CityName', $data['emisor']['ubicacion']['provincia'])
            ->append_nnv('cbc:CountrySubentity', $data['emisor']['ubicacion']['departamento'])
            ->append_nnv('cbc:District', $data['emisor']['ubicacion']['distrito'])
            ->append_fw('cac:Country')->append_nnv('cbc:IdentificationCode', $data['emisor']['ubicacion']['pais'])->pop()
            ->pop()
            ->append_fw('cac:PartyLegalEntity')
            ->append('cbc:RegistrationName', $data['emisor']['datos']['razon_social'], true)
            ->reset()
            ->append_fw('cac:AccountingCustomerParty')
            ->append('cbc:CustomerAssignedAccountID', $data['cliente']['documento']['numero'])
            ->append('cbc:AdditionalAccountID', $data['cliente']['documento']['tipo'])
            ->append_fw('cac:Party');

        if ($data['documento']['tipo_factura'] === '01') {
            $this
                ->append_fw('cac:PartyName')
                ->append('cbc:Name', $data['cliente']['datos']['nombre_comercial'], true)
                ->pop()
                ->append_fw('cac:PostalAddress')
                ->append_nnv('cbc:ID', $data['cliente']['ubicacion']['ubigeo'])
                ->append_nnv('cbc:StreetName', $data['cliente']['ubicacion']['direccion'])
                ->append_nnv('cbc:CitySubdivisionName', $data['cliente']['ubicacion']['urbanizacion'])
                ->append_nnv('cbc:CityName', $data['cliente']['ubicacion']['provincia'])
                ->append_nnv('cbc:CountrySubentity', $data['cliente']['ubicacion']['departamento'])
                ->append_nnv('cbc:District', $data['cliente']['ubicacion']['distrito'])
                ->append_fw('cac:Country')->append_nnv('cbc:IdentificationCode', $data['cliente']['ubicacion']['pais'])->pop()
                ->pop();

        } elseif ($data['documento']['tipo_factura'] === '03') {
            $direccion = $data['cliente']['ubicacion']['direccion'];
            if (!is_null($data['cliente']['ubicacion']['urbanizacion'])) {
                $direccion = $direccion . " - " . $data['cliente']['ubicacion']['urbanizacion'];
            }
            if (!is_null($data['cliente']['ubicacion']['distrito'])) {
                $direccion = $direccion . " - " . $data['cliente']['ubicacion']['distrito'];
            }
            if (!is_null($data['cliente']['ubicacion']['provincia'])) {
                $direccion = $direccion . " - " . $data['cliente']['ubicacion']['provincia'];
            }
            if (!is_null($data['cliente']['ubicacion']['departamento'])) {
                $direccion = $direccion . " - " . $data['cliente']['ubicacion']['departamento'];
            }
            if (!is_null($data['cliente']['ubicacion']['pais'])) {
                $direccion = $direccion . " - " . $data['cliente']['ubicacion']['pais'];
            }
            $this
                ->append_fw('cac:PhysicalLocation')
                ->append('cbc:Description', $direccion, true)
                ->pop();
        }

        $this
            ->append_fw('cac:PartyLegalEntity')
            ->append('cbc:RegistrationName', $data['cliente']['datos']['razon_social'], true)
            ->reset();

        //AGREGAR DATOS DE ANTICIPOS DEVENGADOS
        foreach ($impuestos as $idx => $impuesto) {
            $this
                ->append_fw('cac:TaxTotal')
                ->append('cbc:TaxAmount', $impuesto['monto'])->attribute('currencyID', $data['documento']['moneda'])
                ->append_fw('cac:TaxSubtotal')
                ->append('cbc:TaxAmount', $impuesto['monto'])->attribute('currencyID', $data['documento']['moneda'])
                ->append_fw('cac:TaxCategory')
                ->append_fw('cac:TaxScheme')
                ->append('cbc:ID', $impuesto['id'])
                ->append('cbc:Name', $impuesto['nombre'])
                ->append('cbc:TaxTypeCode', $impuesto['codigo'])
                ->reset();

        }
        $this->append_fw('cac:LegalMonetaryTotal');
        if (isset($data['total']['lineas'])) {
            $this->append('cbc:LineExtensionAmount', $data['total']['lineas'])->attribute('currencyID', $data['documento']['moneda']);
        }

        if (isset($data['total']['descuento'])) {
            $this->append('cbc:AllowanceTotalAmount', $data['total']['descuento'])->attribute('currencyID', $data['documento']['moneda']);
        }

        if (isset($data['total']['cargo'])) {
            $this->append('cbc:ChargeTotalAmount', $data['total']['cargo'])->attribute('currencyID', $data['documento']['moneda']);
        }

        if (isset($data['total']['prepagado'])) {
            $this->append('cbc:PrepaidAmount', $data['total']['prepagado'])->attribute('currencyID', $data['documento']['moneda']);
        }

        if (isset($data['total']['pagable'])) {
            $this->append('cbc:PayableAmount', $data['total']['pagable'])->attribute('currencyID', $data['documento']['moneda']);
        }

        foreach ($items as $idx => $item) {
            $this->reset()
                ->append_fw('cac:InvoiceLine');

            // DATOS DEL ITEM
            // IDENTIFICADOR
            $this->append('cbc:ID', $item['id'])
            // CANTIDAD DE ARTICULOS Y UNIDAD DE MEDIDA
                ->append('cbc:InvoicedQuantity', $item['cantidad'])->attribute('unitCode', $item['unidad'])
            // VALOR DE VENTA SIN IMPUESTOS CONSIDERANDO DESCUENTO
                ->append('cbc:LineExtensionAmount', $item['valor_venta'])->attribute('currencyID', $data['documento']['moneda']);

            // PRECIOS UNITARIOS
            $this->append_fw('cac:PricingReference');

            // CODIGO 01 - Valor de venta unitario
            $this->append_fw('cac:AlternativeConditionPrice')
                ->append('cbc:PriceAmount', $item['precio_unitario']['facturado'])->attribute('currencyID', $data['documento']['moneda'])
                ->append('cbc:PriceTypeCode', '01')
                ->pop();

            // CODIGO 02 - Valor de mercado/referencial en caso de operaciones no onerosas
            if (isset($item['precio_unitario']['referencial'])) {
                $this->append_fw('cac:AlternativeConditionPrice')
                    ->append('cbc:PriceAmount', $item['precio_unitario']['referencial'])->attribute('currencyID', $data['documento']['moneda'])
                    ->append('cbc:PriceTypeCode', '02')
                    ->pop();
            }
            $this->pop();

            // DESCUENTOS DEL ITEM
            if (isset($item['descuento'])) {
                $this
                    ->append_fw('cac:AllowanceCharge')
                    ->append('cbc:ChargeIndicator', "false")
                    ->append('cbc:Amount', $item['descuento'])->attribute('currencyID', $data['documento']['moneda'])
                    ->pop();
            }

            // IMPUESTOS
            // IGV CODIGO 1000, VAT
            $this->append_fw('cac:TaxTotal')
                ->append('cbc:TaxAmount', $item['igv']['monto'])->attribute('currencyID', $data['documento']['moneda'])
                ->append_fw('cac:TaxSubtotal')
                ->append('cbc:TaxAmount', $item['igv']['monto'])->attribute('currencyID', $data['documento']['moneda'])
                ->append_fw('cac:TaxCategory')
                ->append('cbc:TaxExemptionReasonCode', $item['igv']['codigo'])
                ->append_fw('cac:TaxScheme')
                ->append('cbc:ID', '1000')
                ->append('cbc:Name', 'IGV')
                ->append('cbc:TaxTypeCode', 'VAT')
                ->pop()
                ->pop()
                ->pop()
                ->pop();

            // ISC CODIGO 2000,
            if (isset($item['isc']['monto'])) {
                $this->append_fw('cac:TaxTotal')
                    ->append('cbc:TaxAmount', $item['isc']['monto'])->attribute('currencyID', $data['documento']['moneda'])
                    ->append_fw('cac:TaxSubtotal')
                    ->append('cbc:TaxAmount', $item['isc']['monto'])->attribute('currencyID', $data['documento']['moneda'])
                    ->append_fw('cac:TaxCategory')
                    ->append('cbc:TierRange', $item['isc']['codigo'])
                    ->append_fw('cac:TaxScheme')
                    ->append('cbc:ID', '2000')
                    ->append('cbc:Name', 'ISC')
                    ->append('cbc:TaxTypeCode', 'EXC')
                    ->pop()
                    ->pop()
                    ->pop()
                    ->pop();
            }

            // DATOS DEL ITEM
            $this->append_fw('cac:Item');

            // DESCRIPCION DETALLADA
            $this->append('cbc:Description', $item['datos']['descripcion']);

            if (isset($item['codigo'])) {
                // CODIGO DEL ARTICULO
                $this->append_fw('cac:SellersItemIdentification')
                    ->append('cbc:ID', $item['datos']['codigo'])
                    ->pop();
            }
            $this->pop();

            // PRECIO UNITARIO SIN CONSIDERAR IGV NI DESCUENTOS
            $this->append_fw('cac:Price')
                ->append('cbc:PriceAmount', $item['valor_unitario'])->attribute('currencyID', $data['documento']['moneda'])
                ->pop();
            $this->pop();
        }
    }

}
